import NET.worlds.scape.*;

public class PolynomialJump extends Action implements NonPersister {
    private int degree = 2;
    private float[] coefficients = {0, 0, 0};
    private float duration = 1.0f;
    private boolean doBuild = false;
    
    private static int SET_FLOAT_SET_TO = 0;
    private static int SET_FLOAT_DURATION = 1;
    private static int SET_FLOAT_TARGET = 2;
    private static int SET_FLOAT_TARGET_NAME = 4;
    private static int SET_FLOAT_PROPERTY_NAME = 5;
    
    private static int SET_STRING_SET_TO = 0;
    private static int SET_STRING_TARGET = 1;
    private static int SET_STRING_PROPERTY_NAME = 4;
    
    private SmoothDriver getSmoothDriver() {
        return ((HoloPilot)Pilot.getActive()).getSmoothDriver();
    }
    
    public Persister trigger(Event event, Persister persister) {
        // TODO
        try {
            WObject owner = (WObject)getOwner();
            
            SequenceAction sequence = new SequenceAction();
            sequence.setName(getName());
            ParallelAction motions = new ParallelAction();
            SequenceAction resets = new SequenceAction();
            SequenceAction inits = new SequenceAction();
            motions.setWaitForAll();
            
            Action priorAction = null;
            SetFloatAction constant = null;
            float accum = 0.0f;
            
            for(int i = 0; i <= degree; i++) {
                accum = coefficients[i] * (float)Math.pow(duration, i) + accum;
                Action curAction = null;
                SetFloatAction reset = null;
                if(i == 0) {
                    SetFloatAction setFloatAction = new SetFloatAction();
                    curAction = setFloatAction;
                    setFloatAction.properties(SET_FLOAT_SET_TO, 0, Properties.SET, new Float(accum));
                    setFloatAction.properties(SET_FLOAT_TARGET_NAME, 0, Properties.SET, "SmoothDriver1");
                    setFloatAction.properties(SET_FLOAT_TARGET, 0, Properties.SET, getSmoothDriver());
                    setFloatAction.properties(SET_FLOAT_PROPERTY_NAME, 0, Properties.SET, "Eye Height");
                    setFloatAction.properties(SET_FLOAT_DURATION, 0, Properties.SET, new Float(0.0));
                    constant = setFloatAction;
                    
                    SetStringAction init = new SetStringAction();
                    init.properties(SET_STRING_SET_TO, 0, Properties.SET, "SmoothDriver1");
                    init.properties(SET_STRING_TARGET, 0, Properties.SET, setFloatAction);
                    init.properties(SET_STRING_PROPERTY_NAME, 0, Properties.SET, "Target Name");
                    owner.addAction(init);
                    inits.addComponent(init);
                }else if(i == 1) {
                    SetFloatAction setFloatAction = new SetFloatAction();
                    setFloatAction.properties(SET_FLOAT_SET_TO, 0, Properties.SET, new Float(accum));
                    setFloatAction.properties(SET_FLOAT_TARGET_NAME, 0, Properties.SET, "SmoothDriver1");
                    setFloatAction.properties(SET_FLOAT_TARGET, 0, Properties.SET, getSmoothDriver());
                    setFloatAction.properties(SET_FLOAT_PROPERTY_NAME, 0, Properties.SET, "Eye Height");
                    setFloatAction.properties(SET_FLOAT_DURATION, 0, Properties.SET, new Float(duration));
                    curAction = setFloatAction;
                    reset = new SetFloatAction();
                    reset.properties(SET_FLOAT_SET_TO, 0, Properties.SET, new Float(accum));
                    reset.properties(SET_FLOAT_TARGET, 0, Properties.SET, setFloatAction);
                    reset.properties(SET_FLOAT_PROPERTY_NAME, 0, Properties.SET, "Set To");
                    
                    SetStringAction init = new SetStringAction();
                    init.properties(SET_STRING_SET_TO, 0, Properties.SET, "SmoothDriver1");
                    init.properties(SET_STRING_TARGET, 0, Properties.SET, setFloatAction);
                    init.properties(SET_STRING_PROPERTY_NAME, 0, Properties.SET, "Target Name");
                    owner.addAction(init);
                    inits.addComponent(init);
                } else {
                    SetFloatAction setFloatAction = new SetFloatAction();
                    curAction = setFloatAction;
                    setFloatAction.properties(SET_FLOAT_SET_TO, 0, Properties.SET, new Float(accum));
                    setFloatAction.properties(SET_FLOAT_TARGET, 0, Properties.SET, priorAction);
                    setFloatAction.properties(SET_FLOAT_DURATION, 0, Properties.SET, new Float(duration));
                    setFloatAction.properties(SET_FLOAT_PROPERTY_NAME, 0, Properties.SET, "Set To");
                    reset = new SetFloatAction();
                    reset.properties(SET_FLOAT_SET_TO, 0, Properties.SET, new Float(accum));
                    reset.properties(SET_FLOAT_TARGET, 0, Properties.SET, setFloatAction);
                    reset.properties(SET_FLOAT_PROPERTY_NAME, 0, Properties.SET, "Set To");
                }
                if(i != 0) {
                    motions.addComponent(curAction);
                    resets.addComponent(reset);
                }
                owner.addAction(curAction);
                if(i != 0) {
                    owner.addAction(reset);
                }
                priorAction = curAction;
            }
            sequence.addComponent(inits);
            sequence.addComponent(constant);
            sequence.addComponent(motions);
            sequence.addComponent(resets);
            owner.addAction(inits);
            owner.addAction(sequence);
            owner.addAction(motions);
            owner.addAction(resets);
            owner.removeAction(this);
            return null;
        } catch(NoSuchPropertyException e) {
            throw new RuntimeException(e.toString());
        }
    }
    
    
    public Object properties(int index, int offset, int mode, Object value) throws NoSuchPropertyException {
        Object ret = null;
        int what = index - offset;
        
        if(what == 0) {
            if(mode == 0) {
                ret = IntegerPropertyEditor.make(new Property(this, index, "Degree"));
            } else if(mode == 1) {
                ret = new Integer(degree);
            } else if(mode == 2) {
                degree = ((Integer)value).intValue();
                coefficients = new float[degree + 1];
            }
        } else if(what == 1) {
            if(mode == 0) {
                ret = FloatPropertyEditor.make(new Property(this, index, "Duration"));
            } else if(mode == 1) {
                ret = new Float(duration);
            } else if(mode == 2) {
                duration = ((Float)value).floatValue();
            }
        } else if(2 <= what && what <= degree + 2) {
            int coefficient = what - 2;
            if(mode == 0) {
                ret = FloatPropertyEditor.make(new Property(this, index, "t^" + Integer.toString(coefficient) + " coefficient"));
            } else if(mode == 1) {
                ret = new Float(coefficients[coefficient]);
            } else if(mode == 2) {
                coefficients[coefficient] = ((Float)value).floatValue();
            }
        } else {
            ret = super.properties(index, offset + degree + 2 + 1, mode, value);
        }
        return ret;
    }
    
}